package com.sams.jws.chapter15;
/**
 * FileRetrieverServlet
 * Servlet to demonstrate use of JAXM. Calls a remote file
 * server Web service, and renders the results appropriately 
 * for browser consumption.
 */
// General JDK
import java.io.IOException;
import java.util.Iterator;
// JAXM
import javax.xml.soap.SOAPConnectionFactory;
import javax.xml.soap.SOAPConnection;
import javax.xml.soap.MessageFactory;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPPart;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPHeader;
import javax.xml.soap.SOAPHeaderElement;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPBodyElement;
import javax.xml.soap.SOAPFault;
import javax.xml.soap.Name;
import javax.xml.soap.SOAPException;
import javax.xml.soap.AttachmentPart;
import javax.xml.messaging.URLEndpoint;
// JAF
import javax.activation.DataHandler;
// Servlet
import javax.servlet.*;
import javax.servlet.http.*;

public class FileRetrieverServlet extends HttpServlet {
    private final static boolean DEBUG = true;
    private final static String FILE_SERVICE_URL =
        "http://localhost:8080/jaxm/servlet/com.sams.jws.chapter15.FileService";
    public URLEndpoint urlEndpoint = new URLEndpoint(FILE_SERVICE_URL);
    private final static String DEFAULT_TARGET = "/";
    public SOAPConnection soapConnection = null;
    /**
     * doGet()
     * Respond to browser request for file listing from remote FileService. 
     * Render results of retrieval appropriately, either by passing the 
     * file directly to the browser, returning a directory listing if 
     * the file names a directory, or returning an error 
     * page explaining that a problem was encountered.
     * File to list is variable FileName in get query string.
     */
    public void doGet(
        HttpServletRequest httpServletRequest,
        HttpServletResponse httpServletResponse)
        throws ServletException {

        ServletOutputStream outputStream = null;
        try {
            outputStream = httpServletResponse.getOutputStream();

            String targetFile = httpServletRequest.getParameter("FileName");
            if ((targetFile == null) || (targetFile == ""))
                targetFile = DEFAULT_TARGET;

            if (DEBUG)
                System.out.println(
                    "FileRetrieverServlet has retrieval request for file: " + targetFile);

            // Send file retrieval request to FileService.
            SOAPMessage responseSoapMsg = sendRequestMsg(targetFile);

            // Get some utility references parts of response message.
            SOAPEnvelope responseSoapEnv = responseSoapMsg.getSOAPPart().getEnvelope();
            SOAPHeader responseSoapHdr = responseSoapEnv.getHeader();
            SOAPBody responseSoapBody = responseSoapEnv.getBody();

            // Determine response type and render accordingly.
            String responseType =
                DocumentUtils.findValueOfChild(responseSoapHdr, "ResponseType");
            if ((responseType != null) && (responseType.equals("directoryListing"))) {
                // Return message as is to browser. Ultimately, this could be
                // replaced with a transform step that creates an HTML listing 
                // of the directory contents, with appropriate links to each file.
                httpServletResponse.setContentType("text/xml");
                responseSoapMsg.writeTo(outputStream);
            } else if (responseType.equals("attachedFile")) {
                // Return attachment.
                Iterator iterator = responseSoapMsg.getAttachments();
                if (iterator.hasNext()) {
                    // We are assuming that the first attachment part is 
                    // the file we are after. A better solution is to 
                    // look for a particular MIME header.
                    AttachmentPart attachmentPart = (AttachmentPart) iterator.next();
                    DataHandler dataHandler = attachmentPart.getDataHandler();
                    httpServletResponse.setContentType(dataHandler.getContentType());
                    dataHandler.writeTo(outputStream);
                } else {
                    renderErrorPage(outputStream, "Could not locate attached file", targetFile);
                } // Else
            } else if (responseType.equals("fault")) {
                SOAPFault soapFault = responseSoapBody.getFault();
                httpServletResponse.setContentType("text/html");
                renderErrorPage(
                    outputStream,
                    "<B>Code:</B> " + soapFault.getFaultCode(),
                    "<B>Description:</B> " + soapFault.getFaultString());
            } else {
                // Unidentifiable response
                httpServletResponse.setContentType("text/html");
                renderErrorPage(
                    outputStream,
                    "Received corrupted response from server.",
                    "<B>Could not retrieve file:</B> " + targetFile);
            } 
            // General SOAPMessage manipulation exceptions.
        } catch (SOAPException e) {
            System.err.println("Encountered general SOAPException: ");
            e.printStackTrace();
        } catch (IOException e) {
            System.err.println(
                "Encountered IOException when trying to write SOAPMessage: ");
            e.printStackTrace();
        } 
    } // doGet()
    public void init(ServletConfig servletConfig) throws ServletException {
        super.init(servletConfig);
        try {
            // Initialize connection so that all instances can use it.
            SOAPConnectionFactory soapConnFct = SOAPConnectionFactory.newInstance();
            soapConnection = soapConnFct.createConnection();
        } catch (SOAPException e) {
            System.err.println("Encountered general SOAPException: ");
            e.printStackTrace();
        }
    } // init()
    /**
     * renderErrorPage
     * Return an HTML formatted error page to the ServletOutputStream
     */
    public void renderErrorPage(
        ServletOutputStream outputStream,
        String line1,
        String line2)
        throws IOException {
        outputStream.println("<HTML>");
        outputStream.println("Error encountered trying to fullfil request.<P>");
        outputStream.println(line1 + "<P>");
        outputStream.println(line2 + "<P>");
        outputStream.println("</HTML>");
    } // renderErrorPage()
    /**
     * sendRequestMsg
     * Call the FileService remote file access Web service, requesting 
     * a retrieval of the file targetFile. Return SOAPMessage response.
     */
    public SOAPMessage sendRequestMsg(String targetFile)
        throws SOAPException, IOException {

        // Initialize connection and SOAP Profile
        MessageFactory messageFactory = MessageFactory.newInstance();

        SOAPMessage requestSoapMsg = messageFactory.createMessage();

        SOAPPart requestSoapPart = requestSoapMsg.getSOAPPart();
        SOAPEnvelope requestSoapEnv = requestSoapPart.getEnvelope();
        SOAPHeader requestSoapHdr = requestSoapEnv.getHeader();
        SOAPBody requestSoapBody = requestSoapEnv.getBody();

        // Add header field
        Name msgPurposeName = requestSoapEnv.createName("CmdName");
        SOAPHeaderElement msgPurposeElement =
            requestSoapHdr.addHeaderElement(msgPurposeName);
        msgPurposeElement.setMustUnderstand(true);
        msgPurposeElement.addTextNode("FileService");

        // Add namespace-qualified body field
        Name fileName = requestSoapEnv.createName("FileName");
        SOAPBodyElement fileNameElement = requestSoapBody.addBodyElement(fileName);
        fileNameElement.addTextNode(targetFile);

        // Save all changes to message before printing. 
        // Sending will also implictly save changes.
        requestSoapMsg.saveChanges();

        if (DEBUG) {
            System.out.println("FileRetrieverServlet is sending message:");
            requestSoapMsg.writeTo(System.out);
            System.out.println("\n");
        } 

        // Send request and block until reply appears.
        SOAPMessage responseSoapMsg = soapConnection.call(requestSoapMsg, urlEndpoint);

        if (DEBUG) {
            System.out.println("FileRetrieverServlet received response message:");
            responseSoapMsg.writeTo(System.out);
            System.out.println("\n");
        } 

        return responseSoapMsg;

    } // sendRequestMsg()
} // FileRetrieverServlet